@@ -380,8 +380,10 @@ void SpewTui::cumulativeStatistics(capacity_t jobBytesTransferred,
                                    const TimeHack& jobTransferTime,
                                    capacity_t totalBytesRead,
                                    const TimeHack& totalReadTransferTime,
+                                   capacity_t totalReadOps,
                                    capacity_t totalBytesWritten,
                                    const TimeHack& totalWriteTransferTime,
+                                   capacity_t totalWriteOps,
                                    const TimeHack& totalRunTime)
 {
    mStatsWindow->setJobBytesTransferred(jobBytesTransferred);
